function y= powl7(wt,xd,xc,var,b,b0,a,cTotalN,pow,alpha,sides)
%Calculates power of logistic model. In is a parameter vector with the 
%following structure
%  wt m x 1       matrix of weights for each group of descrete covariates
%  xd m x ms      matrix of values for descrete covariates
%  xc m x mn      matrix of means of continuos covariates
%  var m x mn(mn+1)/2 matrix of variance covariance matrix of continuous covariates
%  b  (ms+mn) x 1 matrix of coeficients under HA
%  b0 (ms+mn) x 1 matrix of coeficients under HO
%  a  (ms+mn) x p contrast matrix
%  cTotalN        sample size
%  pow            power  
%  alpha          alpha level
%  sides          if p=1 then sidedness of test(tests for positive beta)
%  if cTotalN>0 and pow=0 find power
%  if cTotalN>0 and pow>0 find multiplyer of a(makes sense for simple a)
%  if cTotalN=0 and pow>0 find sample size
%  otherwise return -2
%  return -1 if array dimensions are wrong
%  return -3 if problems with alpha or sides
%  return -4 if problems with variance covariance matrix
%  return -5 asked for multiplyier when there is more than one contrast
global TOL2
save('c:\powl8.mat');
swt=size(wt);
sxd=size(xd);
sxc=size(xc);
svar=size(var);
sb=size(b);
sb0=size(b0);
sa=size(a);
ms=sxd(2);%number of descrete covariates 
mn=sxc(2);% number of continuous covariates
mm=(ms+mn);%number of covariates
m=swt(1);%number of groups
df=sa(2);
% The following code returns -1 if the input dimensions are wrong.
if (m~=sxd(1) & ms~=0)|(m~=sxc(1) & mn~=0)|(m~=svar(1) & mn~=0)...
     | mm~=min([sb(1) sb0(1) sa(1)]) |  mm~=max([sb(1) sb0(1) a(1)])|...
      svar(2)~=mn*(mn+1)/2
   y=-1 ;
   return;
end
% This code classifies problem or returns error of inputs are wrong
if (cTotalN>0 & pow==0)
   cc=1;
else if (cTotalN>0 & pow>0 & pow<1 & df==1) 
      cc=2;
   else if (cTotalN>0 & pow>0 & pow<1 & df~=1) 
         y=-5;
         return;
      else if (cTotalN==0 & pow>0 & pow<1 )
            cTotalN=1;
            cc=3;
         else
            y=-2; 
            return
         end
      end 
   end
end
% This code checks for other errors
if (alpha<=0) | (alpha>1) | ~(sides==1 | sides==2)
   y=-3;
   return;
end
wt=wt/sum(wt);%weights must sum to 1
bc=b(ms+1:mm);%beta continuous
v=zeros(mm,mm);
xx=[xd xc];
cvs=zeros(mn,mn);
for ii=1:m
   index=0;
   for js=1:mn
      cvs(js,js:mn)=...
         var(ii,(1+mn*(js-1)-(js-1)*(js-2)/2):(mn*(js)-(js)*(js-1)/2));
      cvs(js:mn,js)=cvs(js,js:mn)';
   end
   dt=det(cvs);
   da=min(diag(cvs));
   if dt<=0 | da<=0
      y=-4;
      return;
   end
   us=xx(ii,:)';
   offset=b'*us;
   ncvs=[zeros(ms,ms) zeros(ms,mn);zeros(mn,ms) cvs];
   if any(bc)
      %approximate integrals to 3 sig figures.
      scale=sqrt(bc'*cvs*bc);
      et0=inteval(offset,scale,0);
      et1=inteval(offset,scale,1);
      et2=inteval(offset,scale,2);
      bs=ncvs*b/scale;
      v(:,:)=v(:,:)+cTotalN*wt(ii)*(us*us'*et0+us*bs'*et1+bs*us'*et1+bs*bs'*et2+...
         (ncvs-bs*bs')*et0);
   else
      v(:,:)=v(:,:)+((us*us') + ncvs)*(cTotalN*wt(ii))*exp(offset)/(1+exp(offset))^2;    ;  
   end
end
if sides==1 & df==1
   zz=a'*(b-b0)/sqrt(a'*pinv(v)*a);
else
    zz=((b-b0)'*a)*pinv(a'*pinv(v)*a)*(a'*(b-b0));
   df=rank(a'*pinv(v)*a);
end
crit=norminv(1-alpha);
switch cc
case 1 %Return Power
   if sides==1 & df==1
      y=normcdf(zz-crit);%NEED TO FIND;
   else
      y=1-ncx2cdf(chi2inv(1-alpha,df),df,zz);

   end
case 2 % Return Multiplyer
   if sides==1 & df==1
      y=(norminv(pow)+crit)/zz;
   else
      zz=a'*(b-b0)/sqrt(a'*pinv(v)*a);
      crit=norminv(1-alpha/2);
      y0=(norminv(pow)+crit)/abs(zz);
      %opt=optimset('TolX',0.001);
      opt=optimset;
      y=fzero('fmult1',y0,opt,pow,crit,zz);
   end
case 3 % Return sample size
   if sides==1 & df==1
      y=ceil(((norminv(pow)+crit)/abs(zz) )^2);
   else
      crit=chi2inv(1-alpha,df);
      pp=norminv(pow);
      tmp1=crit-df+2*pp;tmp2=sqrt(2*pp*(2*crit-df+2*pp));
      m1=(tmp1-tmp2)/zz;m2=(tmp1+tmp2)/zz;
      opt=optimset;
      y=ceil(fzero('fmult2',m2,opt,pow,crit,zz,df));
      
   end
   return
end

function y=inteval(offset,scale,i);
global TOL2
err=1;
range=5;
while (err>.001 & range<10)
   %ys=quad8('nint',-range,range,[],[],offset,scale,i);
  ys= quadl('nint',-range,+range,[],[],offset,scale,i);
    switch i
   case 0
      err=normcdf(-range)/abs(ys);
   case 1
      if ys==0; 
         err=0;
      else
         
         err=(exp(-range^2)/sqrt(2*pi))/abs(ys);
      end
   case 2
      err=(.5+ range*exp(-range^2/2)/sqrt(2*pi)-.5*erf(range/sqrt(2)))/abs(ys);
   end
   range=range+1;
end
y=ys;

function y=nint(x,mean,sig,i)
wt=(exp(sig*x+mean))./((1+exp(sig*x+mean)).^2 );
y=(x.^i).*wt.* exp(-x.^2/2)/sqrt(2*pi);
 